<?php

namespace EasyVideoReviews;

defined('ABSPATH') or die('NOT ALLOWED');

# Handles admin hooks
if (!class_exists("\EasyVideoReviews\AdminHooks")) {

    final class AdminHooks
    {
        public $instanced = null;

        # Singleton pattern to avoid multiple instance of this class
        public function __construct()
        {
            if ($this->instanced !== true) {
                $this->instanced = true;
                $this->register_hooks();
            }
        }
 
        # Registers the hooks
        function register_hooks()
        {
            # Triggers duration activation and deactivation only
            register_activation_hook(EVR_FILE, [$this, 'plugin_activated']);
            register_deactivation_hook(EVR_FILE, [$this, 'plugin_deactivated']);

            # Register action hooks
            add_action("admin_enqueue_scripts", [$this, 'admin_enqueue_scripts']);

            add_action("admin_menu", [$this, 'register_admin_menus']);                        
            
            add_action("admin_init", [$this, 'after_installed_evr']);

            add_action("plugin_action_links_" . plugin_basename( EVR_FILE ), [$this, 'evr_action_links']);        
            
            add_action("admin_init", [$this, 'after_x_days']);

            add_action("init", [$this, 'plugin_load_textdomain']);

            add_action("init", [$this, 'load_appsero_client']);

            add_action("admin_footer", [$this, 'admin_footer_scripts']);

            # Register filter hooks
        }

        # Load Easy Video Reviews Languages
        public function plugin_load_textdomain()
        {
            load_plugin_textdomain('easy-video-reviews', false, EVR_PATH . '/languages/');
        }



        # activation hook callback
        function plugin_activated()
        {
            # Resets the settings to default when the plugin is installed for the first time
            $this->reset_evr_options();


            # Creates custom page for WooCommerce integrations
            if (class_exists('\WooCommerce') || class_exists('\Easy_Digital_Downloads')) {
                $this->evr_create_dynamic_page();
            }

            # Managing EVR Clock
            $this->init_install_at();

            evr_log('Easy Video Reviews Activated');
        }

        # Stores the install time of the plugin callback
        public function init_install_at()
        {
            # Stores the install time of the plugin
            $evr_installed_at = get_option('evr_installed_at');
            if (!$evr_installed_at) {
                update_option('evr_installed_at', date("Y-m-d H:i:s"));
            }

            # Resets notice popup time  if no option found
            $evr_notice_review = get_option('evr_notice_review');
            if (!$evr_notice_review) {
                update_option('evr_notice_review', 0);
            }

            # Resets affiliate popup time if no option found
            $evr_notice_affiliate = get_option('evr_notice_affiliate');
            if (!$evr_notice_affiliate) {
                update_option('evr_notice_affiliate', 0);
            }
        }


        # Shows notices after specific days
        public function after_x_days()
        {
            # Resets EVR Clock
            $this->init_install_at();

            $evr_installed_at = get_option('evr_installed_at');

            if ($evr_installed_at) {

                $after7Days = strtotime($evr_installed_at . ' + 7 days');
                $after14Days = strtotime($evr_installed_at . ' + 14 days');

                # execute after 7 days 
                if ($after7Days <= time()) {
                    $this->show_notice_review();
                }

                # execute after 15 days 
                if ($after14Days <= time()) {
                    $this->show_notice_affiliate();
                }
            }
        }

        # Shows admin notice asking for review
        public function show_notice_review()
        {
            $evr_notice_review = (bool) get_option('evr_notice_review', 0);

            if (!$evr_notice_review) {
                add_action('admin_notices', function () {
?>
                    <div class="notice notice-info is-dismissible mb-2 flex items-center p-2" data-evr-notice="evr_notice_review">
                        <h4><span class="dashicons dashicons-heart text-sm"></span><?php _e('Loving Easy Video Reviews? Leave a 5 star rating to help us growing', 'easy-video-reviews'); ?>
                        </h4>
                        <button class="border-0 cursor-pointer bg-blue-400 hover:bg-blue-500 transition duration-100 text-white px-2 py-1 rounded-sm mx-2" data-evr-ajax="1"><?php _e('Rate us', 'easy-video-reviews'); ?></button>
                        <button class="border-0 cursor-pointer bg-red-400 hover:bg-red-500 transition duration-100 text-white px-2 py-1 rounded-sm" data-evr-ajax="0"><?php _e('Don\'t disturb', 'easy-video-reviews'); ?></button>
                    </div>
                <?php
                });
            }
        }

        # Shows admin notice asking for joining affiliate
        public function show_notice_affiliate()
        {
            $evr_notice_affiliate = (bool) get_option('evr_notice_affiliate', 0);
            if (!$evr_notice_affiliate) {
                add_action('admin_notices', function () {
                ?>
                    <div class="notice notice-info is-dismissible mb-2 flex items-center p-2" data-evr-notice="evr_notice_affiliate">
                        <h4><span class="dashicons dashicons-money-alt text-sm"></span><?php _e('Earn 25% of each product sale we make', 'easy-video-reviews'); ?>
                        </h4>
                        <button class="border-0 cursor-pointer bg-blue-400 hover:bg-blue-500 transition duration-100 text-white px-2 py-1 rounded-sm mx-2" data-evr-ajax="1"><?php _e('Join Affiliate', 'easy-video-reviews'); ?></button>
                        <button class="border-0 cursor-pointer bg-red-400 hover:bg-red-500 transition duration-100 text-white px-2 py-1 rounded-sm" data-evr-ajax="0"><?php _e('Don\'t disturb', 'easy-video-reviews'); ?></button>
                    </div>
<?php
                });
            }
        }

        # Creates dynamic pages for WooCommerce
        public function evr_create_dynamic_page()
        {
            // add dynamic page 
            $recorder_page = maybe_serialize(get_option('evr_recorder_page'));
            if (empty($recorder_page)) {
                $post_details = array(
                    'post_title'    => __('Leave Video Review', 'easy-video-reviews'),
                    'post_content'  => '[evr-button]Leave a Review[/evr-button]',
                    'post_status'   => 'publish',
                    'post_author'   => 1,
                    'post_type' => 'page'
                );
                $page_id = wp_insert_post($post_details);
                update_option('evr_recorder_page', $page_id);
            }
        }

        # Redirects to billing page after activation the plugin
        function after_installed_evr()
        {
            $redirected = get_option('evr_redirected', false);
            if ($redirected) return;

            update_option('evr_redirected', true);
            if(is_evr_logged_in()) wp_redirect(admin_url('admin.php?page=easy-video-reviews-billing'));
            else wp_redirect(admin_url('admin.php?page=easy-video-reviews'));
            exit();
        }


        # Executes before the plugin is deactivated
        function plugin_deactivated()
        {
            # deletes authentication token 
            // delete_option('evr_token');
            delete_option('evr_redirected', true);
        }


        # Loads assets for backend dashboard
        function admin_enqueue_scripts($hook)
        {
            $current_screen = get_current_screen();
            $isBlockEditor = method_exists( $current_screen, 'is_block_editor' ) && $current_screen->is_block_editor();
           
 
            if (!$isBlockEditor && !in_array($hook, [
                'toplevel_page_easy-video-reviews',
                'easy-video-reviews_page_easy-video-reviews-settings',
                'easy-video-reviews_page_easy-video-reviews-billing',
                'easy-video-reviews_page_easy-video-reviews-tools',
                'easy-video-reviews_page_easy-video-reviews-integrations',
                'easy-video-reviews_page_easy-video-reviews-profile',
            ])) return false;

            $args_pages = array(
                'sort_order' => 'asc',
                'sort_column' => 'post_title',
                'hierarchical' => 1,
                'post_type' => 'page',
                'post_status' => 'publish',
                'posts_per_page' => -1
            );
            $pages = get_pages($args_pages);
            $allpages = [];
            foreach ($pages as $page) {
                $allpages[] = [
                    'value' => $page->ID,
                    'label' => $page->post_title,
                    'slug' => $page->post_name,
                ];
            }

            $plugin = new \WPPOOL\Product('easy-video-reviews');


            $admin_footer_script = [
                "ajaxurl" => admin_url('admin-ajax.php'),
                "remote" => App::get_remote_url(),
                "plugin_url" => EVR_URL, 
                "home_url" => home_url(), 
                "nonce" => wp_create_nonce('evr_admin'),  
                "host" => $_SERVER['HTTP_HOST'],
                'page' => @end(explode('-', $hook)),
                'extensions' => apply_filters('evr_extensions', []),
                'debug'  => apply_filters('evr_debug', false),
                'offer_data' => $plugin->offer(),                
            ];

            # Conditional elements
            if(is_evr_logged_in()){
                $admin_footer_script['token']       = get_evr_option('token');
                $admin_footer_script['woocommerce'] = true === class_exists('\WooCommerce');
                $admin_footer_script['edd']         = true === class_exists('\Easy_Digital_Downloads');
                $admin_footer_script['pages']       = $allpages;
            } else {
                $admin_footer_script['prefill']     = [
                    'email'         => get_option('admin_email', ''),
                    'first_name'    => get_user_meta(get_current_user_id(), 'first_name', true),
                    'last_name'     => get_user_meta(get_current_user_id(), 'last_name', true),
                ];
            }

            $admin_footer_script = apply_filters('evr_admin_localize_script', $admin_footer_script);

            wp_register_script('_evr_inline', '');
            wp_localize_script('_evr_inline', '_evr_admin', $admin_footer_script);
            wp_enqueue_script('_evr_inline');


            wp_enqueue_script('swal2', EVR_PUBLIC . 'js/sweetalert2.js', [], microtime(), true);

            wp_enqueue_style('evr-admin', EVR_PUBLIC . 'css/admin.min.css');
            wp_enqueue_script('evr-admin', EVR_PUBLIC . 'js/admin.min.js', ['jquery'], microtime(), true);

            wp_enqueue_editor();
            
        } 

        # Loads admin menu pages
        function register_admin_menus()
        {


            if (is_evr_logged_in()) {
                # Logged in

                # Dashboard template
                add_menu_page(__('Easy Video Reviews', 'easy-video-reviews'), __('Easy Video Reviews', 'easy-video-reviews'), 'edit_others_posts', 'easy-video-reviews', function () {
                    evr_render('admin/base', 'reviews/base');
                }, EVR_PUBLIC . '/images/evr.svg');

                # Reviews template
                add_submenu_page('easy-video-reviews', __('Easy Video Reviews Reviews', 'easy-video-reviews'), __('Reviews', 'easy-video-reviews'),  'edit_others_posts', 'easy-video-reviews', function () {
                    evr_render('admin/base', 'reviews/base', 0);
                });

                # Profile template
                add_submenu_page('easy-video-reviews', __('Easy Video Reviews Profile', 'easy-video-reviews'), __('Profile', 'easy-video-reviews'),  'edit_others_posts', 'easy-video-reviews-profile', function () {
                    evr_render('admin/base', 'profile/base');
                });

                # Billing template
                add_submenu_page('easy-video-reviews', __('Easy Video Reviews Billing', 'easy-video-reviews'), __('Billing', 'easy-video-reviews'),  'edit_others_posts', 'easy-video-reviews-billing', function () {
                    evr_render('admin/base', 'billing/base');
                });

                # Integrations template
                add_submenu_page('easy-video-reviews', __('Easy Video Reviews Integrations', 'easy-video-reviews'), __('Integrations', 'easy-video-reviews'),  'edit_others_posts', 'easy-video-reviews-integrations', function () {
                    evr_render('admin/base', 'integrations/base');
                });

                # Tools template
                add_submenu_page('easy-video-reviews', __('Easy Video Reviews Tools', 'easy-video-reviews'), __('Tools', 'easy-video-reviews'),  'edit_others_posts', 'easy-video-reviews-tools', function () {
                    evr_render('admin/base', 'tools/base');
                }, 998);

                # Settings template
                add_submenu_page('easy-video-reviews', __('Easy Video Reviews Settings', 'easy-video-reviews'), __('Settings', 'easy-video-reviews'),  'edit_others_posts', 'easy-video-reviews-settings', function () {
                    evr_render('admin/base', 'settings/base');
                }, 999);

                if (!is_evr_pro()) {
                    add_submenu_page('easy-video-reviews', __('Easy Video Reviews Pro', 'easy-video-reviews'), __('Upgrade <span class="awaiting-mod">Pro</span>', 'easy-video-reviews'),  'edit_others_posts', admin_url('admin.php?page=easy-video-reviews-billing#plans'), '');
                }
                   
            } else {

                # Loads get started page if the user is NOT logged in EVR
                add_menu_page(__('Get Started With Easy Video Reviews', 'easy-video-reviews'), __('Easy Video Reviews', 'easy-video-reviews'), 'edit_others_posts', 'easy-video-reviews', function () {
                    evr_render('get-started/base');
                }, plugin_dir_url(EVR_FILE) . 'public/images/icon.png', 99);
            }
        }

        # Loads recorder template to footer
        function evr_add_script_wp_footer()

        {
            evr_render('recorder');
        }


        # Reset EVR Plugin Options to default
        function reset_evr_options()
        {
            $options = App::get_options();
            foreach ($options as $key => $value) {
                update_option('evr_' . $key, maybe_serialize($value));
            }
        }

        # Load Elementor Widget
        function add_evr_elementor_widgets()
        {
            if(is_evr_pro()){
                require_once EVR_INCLUDES . 'elementor/elementor-buttons.php';
                require_once EVR_INCLUDES . 'elementor/elementor-showcase.php';
                \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new \EasyVideoReviews\EVR_ElementorButtons());
                \Elementor\Plugin::instance()->widgets_manager->register_widget_type(new \EasyVideoReviews\EVR_ElementorShowcase());
            }            
        }


        # On WooCommerce Single Product Tabs
        function evr_woocommerce_product_tabs($tabs)
        {
            $woocommerce_option = get_evr_option('woocommerce');
            if ($woocommerce_option->single_product->button || $woocommerce_option->single_product->showcase) {
                $tabs['easy_video_reviews'] = array(
                    'title'     => __('Video Reviews', 'easy-video-reviews'),
                    'priority'     => 50,
                    'callback'     => [$this, 'evr_custom_data_tab_callback']
                );
            }

            return $tabs;
        }


        # WooCommerce Single Product Tabs Callback Function
        function evr_custom_data_tab_callback()
        {
            global $product;
            $tag = esc_attr("product-" . $product->get_id());

            $woocommerce_option = get_evr_option('woocommerce');
            echo ('<div class="woocommerce">');
            if ($woocommerce_option->single_product->button) {
                echo wp_kses_post(do_shortcode('[evr-button tag=' . $tag . '][/evr-button]'));
            }
            if ($woocommerce_option->single_product->showcase->show) {
                $output = '[evr-videos cols="3" view="grid" limit="' . esc_attr($woocommerce_option->single_product->showcase->limit) . '"  tag=' . esc_attr($tag) . '][/evr-videos]';
                echo wp_kses_post(do_shortcode($output));
            }
            echo '</div>';
        }


        # On WooCommerce Thank you Page
        function evr_woocommerce_thankyou($order_id)
        {
            $woocommerce_option = get_evr_option('woocommerce');
            if (!$woocommerce_option->order_complete->button) return false;
            echo wp_kses_post(do_shortcode('[evr-button tag=' . esc_attr('order-' . $order_id) . ']'));
        }

        # Custom Short-code
        function add_evr_custom_shortcode()
        {
            add_shortcode('evr-button', function ($atts, $content) { {

                    $button = get_evr_option('button');

                    $atts = shortcode_atts(array(
                        'size' => $button->size,
                        'bg' => $button->bg ? $button->bg : 'blue',
                        'color' => $button->color ? $button->color : 'white',
                        'align' => $button->alignment,
                        'tag' => 'general',
                    ), $atts, 'evr-button');


                    $tag = sanitize_text_field($_REQUEST['tag'] ?? false);
                    $tag = empty($tag) ? $atts['tag'] : $tag;
                    $content2 = sanitize_text_field($_REQUEST['button'] ?? false);
                    $content = empty($content2) ? (!empty($content) ? $content : $button->text) : $content2;
                    $out = "<div class='my-4 text-{$atts['align']}'><a href='javascript:;' class='evr-recorder-button' style='color: {$atts['color']}; background: {$atts['bg']}; font-size: {$atts['size']}px' data-evr-button='{$tag}'>{$content}</a></div>";
                    return $out;
                }
            });

            add_shortcode('evr-videos', [$this, 'renderShowcase']);
        }


        # renders video review showcase
        function renderShowcase($atts, $content = null)
        {

            $attributes = \EasyVideoReviews\App::get_showcase_config();
            $attributes["cols"] = "";
            $atts = shortcode_atts($attributes, $atts, 'evr-videos');

            $cols = $attr['cols'] ?? $atts['lgcolumns'];

            $configs = [
                "id" => $atts['id'],
                "tag" => $atts['tag'],
                "view" => $atts['view'],
                "limit" => $atts['limit'],
                "order" => $atts['order'],
                "smcolumns" => $atts['smcolumns'],
                "mdcolumns" => $atts['mdcolumns'],
                "lgcolumns" => $cols,
                "smgap" => $atts['smgap'],
                "mdgap" => $atts['mdgap'],
                "lggap" => $atts['lggap'],
                "pagination" => $atts['pagination'],
                "navigation" => $atts['navigation'],
                "scrollbar" => $atts['scrollbar'],
                "infinity" => $atts['infinity'],
                "autoplay" => $atts['autoplay'],
                "delay" => $atts['delay'],
                "date" => $atts['date'],
            ];

            $showcase = "";

            if ($atts['view'] == 'slider') {

                $showcase = "<div class=\"swiper evr-swiper evr-reviews overflow-x-hidden relative\">
                <div class=\"swiper-wrapper\" data-evr-videos='" . json_encode($configs) . "'\">
                    <div class=\"swiper-slide\">
                        <div class=\"swiper-lazy-preloader\"></div>
                    </div>
                </div>
                " . ($atts['pagination'] == '1' ? "<div class=\"swiper-pagination\"></div>" : "") . "
                " . ($atts['navigation'] == '1' ? " <div class=\"swiper-button-next\"></div>
                <div class=\"swiper-button-prev\"></div>" : "") . ($atts['scrollbar'] == '1' ?  "<div class=\"swiper-scrollbar\"></div>" : "") . "
            </div>";
            } else {

                $showcase = '<div class="evr-reviews relative grid grid-cols-' . esc_attr($cols) . ' md:grid-cols-' . esc_attr($atts["mdcols"]) . ' sm:grid-cols-' . esc_attr($atts["smcols"]) . ' gap-' . esc_attr($atts["lggap"]) . ' md:gap-' . esc_attr($atts["mdgap"]) . ' sm:gap-' . esc_attr($atts["smgap"]) . ' md:gap-3 lg:gap-4" data-evr-videos=\'' . json_encode($configs) . '\'>
                <div class="swiper-lazy-preloader"></div></div>';
            }

            return $showcase;
        }


        #  Single page video page / player
        function evr_custom_single_video_page()
        {
            global $wp_query;
            $optimization = get_evr_option('optimization');

            $query_page_name = isset($wp_query->query_vars['name']) ? $wp_query->query_vars['name'] : (isset($wp_query->query['pagename']) ? $wp_query->query['pagename'] : false);
            if ($query_page_name != ($optimization->showcase->slug ? $optimization->showcase->slug : $optimization['showcase']['slug'])) return false;
            include_once plugin_dir_path(EVR_FILE) . 'templates/single-review.php';
            exit();
        }



        # On WooCommerce Thank you Page
        function build_dynamic_message($pre_text = '', $args = [])
        {

            $output = $pre_text;

            if ($args && !empty($args)) {
                foreach ($args as $arg => $value) {
                    $output = preg_replace("/\{$arg\}/i", $value, $output);;
                }
            }

            return $output;
        }



        # Email message
        function getEmailAppendMessage()
        {
            $email = get_evr_option('email');
            $recorder_page = get_evr_option('recorder_page');

            $url = get_permalink($recorder_page) . '?tag=orders&source=email';

            $message = $this->build_dynamic_message("<div style=\"padding-top:15px; font-family:\"Helvetica Neue\",Helvetica,Roboto,Arial,sans-serif;\"><div style=\"text-align: center;\">{$email->message}</div></div>", [
                'button' => "<div style=\"text-align:center; padding-top: 8px; margin-top: 5px; \">
            <a href=\"{$url}\" target=\"_blank\" style=\"background: {$email->button->bg}; color: {$email->button->color}; font-size: {$email->button->size}px; padding: 10px 12px; border-radius: 5px; text-decoration: none;\">
                {$email->button->text}
            </a>
        </div>"
            ]);

            return apply_filters('evr_thankyou_email_html', $message);
        }



        # WooCommerce Email footer
        function evr_woocommerce_email_footer($email)
        {
            if ($email->id != 'customer_completed_order') return false;
            $woocommerce_option = get_evr_option('woocommerce');
            if (!$woocommerce_option->order_complete->email) return false;

            echo wp_kses_post($this->getEmailAppendMessage());
        }



        # Easy Digital Downloads Button after purchasing
        function evr_edd_add_review_button()
        {
            $edd_option = get_evr_option('edd');

            if (!$edd_option->order_complete->button) return false;
            $button = get_evr_option('button');
            echo wp_kses_post(do_shortcode('[evr-button tag="edd-review"]'));
        }



        # Easy Digital Downloads Email Footer
        function evr_edd_email_footer()
        {
            $edd_option = get_evr_option('edd');
            if (!$edd_option->order_complete->email) return false;

            echo wp_kses_post($this->getEmailAppendMessage());
        }




        # Single Video Page Styles
        function add_styles_to_single_video_page()
        {
            if (!is_admin()) {
                wp_enqueue_style('evr_single_style', EVR_PUBLIC . "css/frontend.css", [], null, false);
            }
        }




        # Single Video Page Scripts
        function add_scripts_to_single_video_page()
        {
            if (!is_admin()) {
                wp_enqueue_script('jquery');
                wp_enqueue_script('evr_single_sc ript', EVR_PUBLIC . "js/frontend.js", [], null, false);
            }
        }


        # Custom Plugin Action Link
        function evr_action_links($links)
        {
            $reviews_link = '<a href="' . esc_url(add_query_arg('page', 'evr-admin/#reviews', get_admin_url() . 'admin.php')) . '">' . __('Reviews', 'easy-video-reviews') . '</a>';
            $setting_link = '<a href="' . esc_url(add_query_arg('page', 'evr-admin/#settings', get_admin_url() . 'admin.php')) . '">' . __('Settings', 'easy-video-reviews') . '</a>';
            
             
            $pro_link = '<a style="color: #e74c3c; font-weight: 500" href="' . esc_url(admin_url( 'admin.php?page=easy-video-reviews-billing#plans' )) . '">' . __('Upgrade to Pro', 'easy-video-reviews') . '</a>';
 
            if (is_evr_logged_in()) {
                array_unshift($links, wp_kses_post($setting_link));
                array_unshift($links, wp_kses_post($reviews_link));
            }
            if (is_evr_pro()) {
                array_push($links, wp_kses_post($pro_link));
            }

            return $links;
        }



        /**
         * Elementor before scripts add custom icon class 
         */
        function evr_elementor_before_enqueue_scripts()
        {
            echo '<style>';
            echo '
        .evr-logo-icon {
        width: 25px;
        height: 20px;
        display: inline-block;
        margin: auto;
        background-image: url(\'' . EVR_PUBLIC . 'images/evr.svg\') !important;
        background-size: 100%;
        }';
            echo '</style>';
        }

        function kses_filter_allowed_html($allowed, $context)
        {
            if (is_array($context)) {
                return $allowed;
            }

            if ($context === 'post') {
                $allowed['a']['data-*'] = true;
                $allowed['table']['data-*'] = true;
            }

            return $allowed;
        }


        # Init appsero client plugin 
        function load_appsero_client()
        {
            if (class_exists('\EasyVideoReviews\Appsero\Client')) { 
                $client = new \EasyVideoReviews\Appsero\Client('98bea24e-5324-4ccf-932a-6f1c77b67ad6', 'Easy Video Reviews', EVR_FILE);
                $client->set_textdomain('easy-video-reviews');
                $client->insights()->init(); 
            }
        }

        # Admin footer script 
        
        function admin_footer_scripts(){
            evr_render('admin/other/admin_footer_scripts');
        }

    }

    # Initialization
    if( is_admin() ) new AdminHooks();
}
